#!/usr/bin/env python

# Works out where the gaps are in the Wikipedia ESC Category pages that are meant to
# list all the countries that competed in a given year


import json
import os
import re

import argparse

import escwikipedia
import util

def gs_fileset_country_in_year(country_year_name, country_year_rec, nul_output_dir_name):

    metadata_array = []
    
    country_year_id = re.sub(r'\s+', '', country_year_name) 

    metadata_array.append({"name": "Identifier", "content": country_year_id})
        
    for metadata_key in country_year_rec.keys():
        metadata_val = country_year_rec.get(metadata_key)
        metadata_array.append({"name": metadata_key, "content": metadata_val})


    # id encodes country and year,
    filename_id = country_year_id + "\\.nul"

    nul_filename = os.path.join(nul_output_dir_name,country_year_id+".nul");

    print("  Creating: " + nul_filename)
        
    with open(nul_filename, 'w') as outfile:
        outfile.write("")
    
    fileset = {
        "FileSet" : [            
            { "FileName": filename_id, },
            { "Description" : { "Metadata" : metadata_array } }
        ]
    }
              
    return fileset



def gs_directory_metadata(missing_year_countries, json_output_filename):


    nul_output_dir_name = os.path.dirname(json_output_filename)
    
    directory_metadata = []

    for year in missing_year_countries.keys():
        year_str = str(year)
        
        for country in missing_year_countries[year]:

            country_year_name = country + year_str
            country_year_rec = missing_year_countries[year][country]
            
            # dig out metadata rec based on year and country
            
            fileset = gs_fileset_country_in_year(country_year_name, country_year_rec, nul_output_dir_name)
            directory_metadata.append(fileset)
        

    greenstone_metadata_json = { "DirectoryMetadata": directory_metadata }

    return greenstone_metadata_json



def save_greenstone_json_metadata(greenstone_metadata_json,json_output_filename):

    with open(json_output_filename, 'w') as outfile:
        json.dump(greenstone_metadata_json, outfile, indent=2)
    

def generate_sparql_query(sparql_values_output_filename,missing_year_countries):

    sparql_values_template_filename = sparql_values_output_filename + ".in"

    missing_uri_lines = []
    
    for year in missing_year_countries.keys():
        year_str = str(year)
        for country in missing_year_countries[year]:
            # Example:
            #  dbr:Cyprus_in_the_Eurovision_Song_Contest_2010
            
            country_wikisafe = country.replace(" ","_")
            
            missing_uri = "      (dbr:" + country_wikisafe + "_in_the_Eurovision_Song_Contest_" + year_str+")"
            
            missing_uri_lines.append(missing_uri)
            
            
    missing_uri_lines_text = "\n".join(missing_uri_lines)
    util.write_text_file_from_template(sparql_values_template_filename,"**missing-country-year-uris**",missing_uri_lines_text,sparql_values_output_filename)

    
if __name__ == "__main__":

    # Sample ESC page for year:
    #   https://en.wikipedia.org/wiki/Eurovision_Song_Contest_1988
    # Sample (buggy!!) ESC *Category* page for that year:
    #   https://en.wikipedia.org/wiki/Category:Countries_in_the_Eurovision_Song_Contest_1988

    
    # 1. Download Wikipedia ESC article pages (about ESC in a given year)
    #    (This looks to have been carefully vetted over time concering countries competeting)
    
    # 2. Download Wikipedia ESC *Category* pages (listscountries in a given year)
    #    (This appears to be at times incomplete. As it the Category pages that
    #     are used as the hook used in the SPARQL query produce the list of all ESC entries
    #     using this approach alone will lead to countries being overloooked)
    
    # 3. Extract list of countries from (1)
    # 4. Extract list of countries from (2)
    
    # 5. Compare the ESC country list against the Catogory list

    
    parser = argparse.ArgumentParser()
    parser.add_argument('--startyear', type=int, default=1956)
    parser.add_argument('--endyear',   type=int, default=2019)
    parser.add_argument('--cachedir', default="cache-wikipedia")
    parser.add_argument('--queryfile.sparql', default=None)
    parser.add_argument('output-file.json', nargs="?", default="metadata.json");
    
    args = parser.parse_args()

    start_year = getattr(args,'startyear')
    end_year   = getattr(args,'endyear')

    cache_dir = getattr(args,'cachedir')
    escwikipedia.init_cache(cache_dir)
    
    sparql_values_output_filename  = getattr(args,'queryfile.sparql')

    json_output_filename  = getattr(args,'output-file.json')


    final_result_ids = [ 'Grand_final', 'Final', 'Results' ]
    semifinal_result_ids = [ 'Semi-final']
    semifinal_pair_result_ids = [ 'Semi-final_1', 'Semi-final_2' ]

    missing_year_countries = {}
    
    for year in range(start_year, end_year+1):
        print("==========")

        article_country_year_html = escwikipedia.retrieve_article_page(year)
        article_final_country_year_recs = escwikipedia.process_article_page(article_country_year_html,year,final_result_ids, stop_at_first=True)
        article_semifinal_country_year_recs = escwikipedia.process_article_page(article_country_year_html,year,semifinal_result_ids, stop_at_first=True)
        if (len(article_semifinal_country_year_recs.keys()) == 0):
            article_semifinal_country_year_recs = escwikipedia.process_article_page(article_country_year_html,year,semifinal_pair_result_ids, stop_at_first=False)
        article_country_year_recs = util.merge_dicts(article_final_country_year_recs,article_semifinal_country_year_recs)

        article_countries = {}
        for country_year in article_country_year_recs.keys():
            article_country = re.search(r"(^.*)\d{4}(?:\w+)?",country_year).group(1)
            
            article_countries[article_country] = article_country_year_recs[country_year]
                
        category_countries = escwikipedia.process_category_page(year)

        ####
        # Work out list "article" - "category"
        ####

        for cat_country in category_countries.keys():
            if (cat_country in article_countries):
                del article_countries[cat_country]
            else:
                print("Country listed in Category page that did not reach Final/Semifinal: " + cat_country)
                
        # Missing countries are the ones left in 'article_countries'
        if (len(article_countries.keys()) == 0):
            print("No countries missing in ESC Category for " + str(year))
        else:
            print("++++++++++")
            print("Countries missing ESC Category for "+str(year)+": "
                  + ", ".join(article_countries.keys()))
            print("++++++++++")

            missing_year_countries[year] = article_countries

        print("==========")
        print()

    print()

    greenstone_metadata_json = gs_directory_metadata(missing_year_countries,json_output_filename)
    print("Saving output as: " + json_output_filename)
    save_greenstone_json_metadata(greenstone_metadata_json,json_output_filename)
    
    if (sparql_values_output_filename is not None):
        generate_sparql_query(sparql_values_output_filename,missing_year_countries)    
        print()
